
#include <stdio.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/time.h>
#ifdef POSIX
#  include <sys/select.h>
#endif
#include <string.h>
#include <ctype.h>
#include "utility.h"

extern void log_msg();

extern time_t time();

char *
skipspace(buf)
    char *buf;
{
    while (isspace(*buf) && (*buf != (char) 0))
	buf++;
    return buf;
}

char *
skipdigits(buf)
    char *buf;
{
    while (isdigit(*buf) && (*buf != (char) 0))
	buf++;
    return buf;
}

#ifdef NEED_STRERROR
char *
strerror(num)
    int num;
{
    static char buf[2048];
    (void) sprintf(buf,"Errno: %d\n",num);
    return buf;
}
#endif

#ifdef NEED_STRSTR
char *
strstr(s1,s2)
    char *s1;
    char *s2;
{
    char *tmp;

    do {
	tmp = strchr(s1,*s2);
	if (!tmp)
	    break;
	if (!strncmp(tmp,s2,strlen(s2)))
	    return tmp;
	tmp++;
    } while (!tmp);

    return (char *) 0; 
}
#endif


void
log_error(prefix, file, lineno)
    char *prefix;
    char *file;
    int lineno;
{
    int loc_errno = errno;
    char logbuf[1024];

    if (loc_errno != 0) {
	(void) sprintf(logbuf, "%s: %s[%s:%d]", prefix, strerror(loc_errno),
		       file, lineno);
	log_msg(logbuf);
    }
}

char *
expand_hilite(mask, buffer)
    int mask;
    char *buffer;
{
    char *cp;
    int bit;

    (void) strcpy(buffer, "[");
    cp = buffer + 2;
    bit = 0;
    while (bit < 8) {
	if ((1 << bit) & mask) {
	    if (cp != (buffer + 2)) {
		*cp = ';';
		cp++;
	    }
	    switch (bit) {
	    case BOLD:
		*cp = '1';
		break;
	    case UNDERLINE:
		*cp = '4';
		break;
	    case BLINK:
		*cp = '5';
		break;
	    case REVERSE:
		*cp = '7';
		break;
	    default:
	    {
		char errbuf[100];
		(void) sprintf(errbuf,
			       "Unknown bit in hilite mask 0x%x number %d",
			       mask, bit);
		log_msg(errbuf);
	    }
		return (char *)0;
	    }
	    cp++;
	}
	bit++;
    }
    *cp = 'm';
    cp[1] = '\000';
    return buffer;
}

int
construct_mask(args, mask)
    char *args;
    int *mask;
{
    int change=0x0;

    while (isspace(*args) && *args)
	args++;
    if (!*args) {
	if (*mask)
	    *mask = 0;
	else
	    *mask = (1 << BOLD);
	return 0;
    }
    while (*args) {
	if (isspace(*args)) {
	    args++;
	    continue;
	}
	switch (*args) {
	default:
	    return -1;
		/*NOTREACHED*/
	    break;
	case 'B':
	    change |= (1 << BLINK);
	    break;
	case 'b':
	    change |= (1 << BOLD);
	    break;
	case 'r':
	    change |= (1 << REVERSE);
	    break;
	case 'u':
	    change |= (1 << UNDERLINE);
	    break;
	case '+':
	    *mask |= change;
	    change = 0x0;
	    break;
	case '-':
	    *mask &= ~change;
	    change = 0x0;
	    break;
	case '=':
	    *mask = change;
	    change = 0x0;
	    break;
	}
	args++;
    }
    return 0;
}

char *
mask2string(mask, buffer, strings, separator)
    int mask;
    char *buffer;
    char **strings;
    char *separator;
{
    int bit;

    bit = 0;
    *buffer = '\000';
    while (bit < 8) {
	if ((1 << bit) & mask) {
	    if (*buffer)
		(void) strcat(buffer, separator);
	    (void) strcat(buffer, strings[bit]);
	}
	bit++;
    }
    return buffer;
}

char *
timelet(idle, precision)
    long idle;
    long precision;
{
    static char buf[81];
    char *cp;
    long timenoc;
    time_t tmptime;
    int wks, days, hrs,mins,secs;
    
    (void) time(&tmptime);
    timenoc = tmptime - idle;
    secs = timenoc;
    mins = secs / 60;
    hrs = mins / 60;
    days = hrs / 24;
    wks = days / 7;
    secs %= 60;
    mins %= 60;
    hrs %= 24;
    days %= 7;
	/* if we ever need to count months....
	   wks %= 4;
	*/

    cp = buf;
    *buf = '\000';
    if (precision && wks) {
	(void) sprintf(cp, "%dw", wks);
	precision--;
	cp = (char *) index(buf, '\000');
    }
    if (precision && days) {
	(void) sprintf(cp, "%dd", days);
	precision--;
	cp = (char *) index(buf, '\000');
    }
    if (precision && hrs) {
	(void) sprintf(cp, "%dh", hrs);
	precision--;
	cp = (char *) index(buf, '\000');
    }
    if (precision && mins) {
	(void) sprintf(cp, "%dm", mins);
	precision--;
	cp = (char *) index(buf, '\000');
    }
    if (precision && secs) {
	(void) sprintf(cp, "%ds", secs);
	precision--;
	cp = (char *) index(buf, '\000');
    }

    return buf;
}

#ifndef LORIEN
#if defined(sun) || defined(sparc) || defined(hpux)
#include <sys/resource.h>

int
getdtablesize()
{
    struct rlimit maxdescs;

    getrlimit(RLIMIT_NOFILE, &maxdescs);

    return (int) maxdescs.rlim_cur;
}

#endif
#endif
